/*********************************************************************/
/*                                                                   */
/*            IBM MQSeries Integrator XML to MRM Utility             */
/*                                                                   */
/*  NAME:           XML2MRM.REX                                      */
/*                                                                   */
/*  DESCRIPTION:    Build C or Cobol map of an XML message           */
/*                  suitable for use with MQSI's MRM                 */
/*                                                                   */
/*********************************************************************/
/*  Licensed Materials - Property of IBM                             */
/*                                                                   */
/*  MQSeries SupportPac MAI5                                         */
/*  (C) Copyright IBM Corporation 2000.                              */
/*                                                                   */
/*  Status: Version 1 Release 1.2                 13 Nov 2001        */
/*********************************************************************/
/*  Summary of Changes:                                              */
/*   V1.1  Corrected problems with Cobol output                      */
/*         Correct handling of Tabs within input file                */
/*         Added report option                                       */
/*   V1.2  Added support for XML Attributes and XML Comments         */       
/*   V1.3  Changed to use a length of 1 for all sizes to prevent     */
/*         duplicate elements.                                       */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*  FUNCTION:       Create a C header or COBOL Copybook based on     */
/*                  an XML structure to be used with the MQSI        */
/*                  Import function.                                 */
/*                                                                   */
/*  PROCESSOR:      REXX                                             */
/*                  Tested with IBM ObjRexx on Windows NT & 2000     */
/*               http://www.software.ibm.com/ad/obj-rexx/            */
/*                  and Regina Rexx V2.0 on Windows NT               */
/*               http://www.lightlink.com/hessling/Regina/Index.html */ 
/*                                                                   */
/*********************************************************************/
 
Address COMMAND;Trace 'O'
/*-------------------------------------------------------------------*/
/* Input:                                                            */
/*       Required: file name of XML message.                         */
/*       Optional: Name of output file                               */
/*       Optional: Output type: C or CObol                           */
/*-------------------------------------------------------------------*/      
Arg InputFileName OutputFileName OutputType .
If InputFileName = '?' | InputFileName = '' Then 
  Do
    Call Help
    Exit 4
  End
/*-------------------------------------------------------------------*/ 
/* Set the Default for OutputType to be C                            */
/*-------------------------------------------------------------------*/ 
OutputType_Default = 'C'
If OutputType = '' Then OutputType = OutputType_Default
Parse Value InputFileName with InputFileNamex '.' ext

If OutputFileName = '' Then 
   OutputFileName= strip(InputFileNamex) ||'.'|| OutputType
/*-------------------------------------------------------------------*/ 
/* If Output File Exists, erase it first                             */
/*-------------------------------------------------------------------*/ 
If Lines(OutputFileName) > 0 THEN
   DO
      Call LineOut(OutputFileName)     
      Address CMD '@ERASE' OutputFileName   
   END

Say "XML2MRM V1.2 - Processing" InputFileName "to create" OutputFileName "for" OutputType

OutputReportFileName= strip(InputFileNamex) ||'_'|| OutputType ||'.RPT'
/*-------------------------------------------------------------------*/ 
/* If Output Report File Exists, erase it first                      */
/*-------------------------------------------------------------------*/ 
If Lines(OutputReportFileName) > 0 THEN
   DO
      Call LineOut(OutputReportFileName)     
      Address CMD '@ERASE' OutputReportFileName   
   END

Say "Report file is" OutputReportFileName 
/*-------------------------------------------------------------------*/ 
/* Convert XML structure to a single line and convert any tab chars  */
/*-------------------------------------------------------------------*/ 
Parse Value '' with single_line sin_line
Do WHILE Lines(InputFileName) >< 0 
      sin_line=LineIn(InputFileName)
      sin_line=translate(sin_line,'20'x,'05'x)   /* strip  tab chars */
      sin_line=translate(sin_line,'20'x,'09'x)   /* strip  tab chars */
      single_line=single_line  strip(sin_line)
END
single_line=strip(single_line)
If left(single_line,2)='<?' Then
   Parse value single_line with '<?' vhdr '?>' single_line
/*-------------------------------------------------------------------*/ 
/* Report the size of the compressed, single line structure          */
/*-------------------------------------------------------------------*/ 
report_line = InputFileName 'XML structure size =' length(single_line)
Say report_line 
wc=Report()
report_line = single_line
wc=Report()
/*-------------------------------------------------------------------*/ 
/* Invoke structure processor and create output                      */
/*-------------------------------------------------------------------*/ 
If OutputType = 'C' Then rc=CCode()
Else rc=Cobol()
wc= Create()
Exit wc
/*-------------------------------------------------------------------*/ 
/* Create C structure                                                */
/*-------------------------------------------------------------------*/ 
CCode: 
    Parse Value '' with new_l. level. trail. aname.
    Level.1=Left(' ',2)
    Level.2=Left(' ',5)
    Level.3=Left(' ',8)
    Level.4=Left(' ',11)
    Level.5=Left(' ',14)
    Level.6=Left(' ',17)
    Level.7=Left(' ',20)
    Level.8=Left(' ',23)
    Level.9=Left(' ',26)
    Parse Value '1 2 1 0 1'  with recnum t level tr one_time
    K=length(single_line)
    addpic='[nn];'
    /*---------------------------------------------------------------*/
    /* Construct first and last lines                                */
    /*---------------------------------------------------------------*/                      
    new_l.1='struct' strip(InputFileNamex) || '_xml2mrm {'
    last2  ='}' strip(InputFileNamex) || ';'  
    /*---------------------------------------------------------------*/
    /* Parse XML                                                     */
    /*---------------------------------------------------------------*/     
    Do Until Length(single_line)=0
       Parse Value single_line with Data '<' name attributes '>' single_line
       single_line=strip(single_line)
       If left(name,1) = '!' Then Iterate
       If one_time=1 Then
         Do
           one_time=0
           new_l.2  =level.1 'struct' name||'_t {'
           last1    =level.1 '}' name';'
           aname.1 = name
           Iterate
         End
       call attributes
       If left(name,1) = '/' then
          Do
            level=level-1
            Parse Value name with '/' checkname
            If checkname <> firstname Then 
              Do         
                t=t+1
                new_l.t=trail.tr
                trail.tr=''
                tr=tr-1
              End
          End
       Else
          Do
            t=t+1
            level=level+1
            c_name=name
            aname.level=name 
            parse value single_line with ItemData '</' nextname . '>' .
            If length(strip(ItemData))=0 & name=nextname Then single_line='?'||single_line
            If left(single_line,1)='<' then
              Do
                tr=tr+1
                new_l.t   =level.level ' struct' c_name||'_t {' 
                trail.tr  =level.level ' }' c_name';'
              End
            Else
              Do
                firstname = name
                rc=assist()
/*              new_l.t   =left(level.level 'char' c_name,50) '['||length(ItemData)||'];'*/
/*V1.3 change */
                new_l.t   =left(level.level 'char' c_name,50) '[1];'
              End
          End
    End
    t=t+1
    new_l.t=last1           /* Add closing setting                   */
    t=t+1
    new_l.t=last2           /* Add closing setting                   */
Return 0
/*-------------------------------------------------------------------*/
/* Record Information useful for ESQL Coding                         */
/*-------------------------------------------------------------------*/
Assist:
    a_line=''
    Do A=1 to Level
       a_line=a_line || '.' || aname.A
    End
    report_line = 'xxxxRoot.XML' || a_line || ';'
    wc=Report()
Return 0
/*-------------------------------------------------------------------*/
/* Turn any XML attributes into xml tags                             */
/*-------------------------------------------------------------------*/
Attributes:
       If length(attributes)=0 Then Return 0
       Parse value 0 with A atname. adata. new_line trail_line
       If Length(attributes)>0 Then 
         Do Until Length(attributes)=0
           A=A+1 
           Parse Value attributes with atname.A '=' '"' adata.A '"' attributes
           atname.A=strip(atname.A)
           if length(atname.A)>0 Then
             Do
               adata.A=strip(adata.A)
               If Length(adata.A)=0 Then adata.A='????'
               If attributes = '/' Then
                 Do
                   attributes=''
                   trail_line='</'name'>'
                 End
             End
           Else A=A-1
        End

       If A>0 Then Do K=1 to A
          new_line = new_line||'<'||atname.K||'>'||adata.K||'</'||atname.K||'>'      
       End
       single_line=new_line||trail_line||single_line
Return 0

/*-------------------------------------------------------------------*/
/* Basic Help                                                        */
/*-------------------------------------------------------------------*/
Help:
Say 'XML2MRM'
Say '         Create a C structure and/or COBOL Copybook based on an'
Say '         XML message.'
Say ''
say ' Input:'                                                            
say '       Required: file name of XML message.'                        
say '       Optional: Name of output file'                       
say '       Optional: Output type: C or CObol'    
Return 4                
/*-------------------------------------------------------------------*/  

/*-------------------------------------------------------------------*/ 
/* Create COBOL Copybook                                             */
/*-------------------------------------------------------------------*/ 
Cobol: 
    Parse Value '' with new_l. 
    Parse Value '1 4 0' with recnum t level
    new_l.=''
    Level.0='       01 '
    Level.1='          05 '
    Level.2='            10 '
    Level.3='              15 '
    Level.4='                20 '
    Level.5='                  25 '
    Level.6='                   30 '
    Level.7='                    35 '
    Level.8='                     40 '
    Level.9='                      41 '
    Level.10='                      41 '
    Level.11='                       42 '
    Level.12='                        43 '
    Level.13='                         44 '
    Level.14='                          45 '
    Level.15='                           46 '
    Level.16='                            47 '
    Level.17='                             48 '
    Level.18='                              49 '
    Level.19='                               50 '
    Level.20='                                51 '

    header1='      ***************************************************************'
    header2='      * COBOL DECLARATION FOR '
    new_l.1=header1
    new_l.2=left(header2||InputFileName,70) '*'
    new_l.3=header1
    new_l.4=level.0 strip(InputFileNamex)||'_XML.'
    K=length(single_line)
    addpic='PIC X(nn).'
    /*---------------------------------------------------------------*/
    /* Parse XML                                                     */
    /*---------------------------------------------------------------*/     
    Do Until Length(single_line)=0
       Parse Value single_line with Data '<' name attributes '>' single_line
       single_line=strip(single_line)
       If left(name,1) = '!' Then Iterate
       Call Attributes
       If left(name,1) = '/' then
          Do
            level=level-1
          End
       Else
          Do
            t=t+1
            level=level+1
            cobol_name=name
            aname.level=name
            parse value single_line with ItemData '</' nextname . '>' .
            If length(strip(ItemData))=0 & name=nextname Then
              Do 
                single_line='?'||single_line
              End
            If left(single_line,1)='<' then
              Do
                new_l.t=level.level cobol_name||'.'
              End
            Else
              Do            
/*              new_l.t=left(level.level cobol_name,50)||'PIC X('||length(ItemData)||').'*/
/* V1.3 Change */
                new_l.t=left(level.level cobol_name,50)||'PIC X('1').'
                aname.level=name
                rc=assist()
              End
           End
    End
Return 0
/*-------------------------------------------------------------------*/
/* Write the results                                                 */
/*-------------------------------------------------------------------*/
Create: 
    Do W=1 to T
        Call LineOut OutputFileName, new_l.W
    End
    Say OutputFileName 'created with' T 'records'
Return 0
/*-------------------------------------------------------------------*/
/* Write to the report file                                          */
/*-------------------------------------------------------------------*/
Report: 
    Call LineOut OutputReportFileName, report_line
Return 0